/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ca.uwo.csd.nlp;

import ca.uwo.csd.nlp.cmplx.CmplxStruct;
import ca.uwo.csd.nlp.cmplx.Disjunction;
import ca.uwo.csd.nlp.pred.Predicate;
import java.util.ArrayList;

/**
 * <code>DRS</code> represents a Discourse Representation Structure. It stores a
 * DRS produced by Attempto Parser Engine in an internal form. This form is the
 * following:
 * DRS = Domain + [Conditions]
 * where Domain  = [Referent] and
 * Condition = [(Predicate | ComplexStructure)]
 * This is a recursive data structure. A DRS can contain nested DRS through
 * ComplexStructure.
 *
 * @author Syeed Ibn Faiz
 */
public class DRS {
    private Domain domain;
    private ArrayList<Condition> conditions;

    /**
     * Constructs an empty DRS.
     */
    public DRS() {
        domain = null;
        conditions = null;
    }

    /**
     * Constructs a DRS from a given domain and a list of conditions.
     * @param domain a domain of discourse
     * @param conditions a set of conditions
     */
    public DRS(Domain domain, ArrayList<Condition> conditions) {
        this.domain = domain;
        this.conditions = conditions;
    }

    public ArrayList<Condition> getConditions() {
        return conditions;
    }

    public Domain getDomain() {
        return domain;
    }

    /**
     * Returns predicate inside this DRS with the given type
     * @param type
     * @return
     */
    public ArrayList<Predicate> getPredicatesByType(String type) {
        ArrayList<Predicate> list = new ArrayList<Predicate>();
        for (Condition c: conditions) {
            if (c instanceof Predicate) {
                Predicate p = (Predicate) c;
                if (p.getType().equals(type)) list.add((Predicate) c);
            }
        }
        return list;
    }
    /**
     * Returns all predicates in this DRS and inside the nested ones with a given type
     * @param type
     * @return
     */
    public ArrayList<Predicate> getAllPredicatesByType(String type) {
        ArrayList<Predicate> list = new ArrayList<Predicate>();
        for (Condition c: conditions) {
            if (c instanceof Predicate) {
                Predicate p = (Predicate) c;
                if (p.getType().equals(type)) list.add((Predicate) c);
            } else {
                if (c.getType().equals("DISJ")) {
                    list.addAll(((Disjunction) c).getDrs1().getAllPredicatesByType(type));
                    list.addAll(((Disjunction) c).getDrs2().getAllPredicatesByType(type));
                } else {
                    list.addAll(((CmplxStruct) c).getNestedDRS().getAllPredicatesByType(type));
                }
            }
        }
        return list;
    }
    public ArrayList<Predicate> getAllPredicatesByRefAndType(String ref, String type) {
        ArrayList<Predicate> list = new ArrayList<Predicate>();
        for (Condition c: conditions) {
            if (c instanceof Predicate) {
                Predicate p = (Predicate) c;
                if (p.getRef().equalsIgnoreCase(ref) && p.getType().equalsIgnoreCase(type)) list.add((Predicate) c);
            } else {
                if (c.getType().equals("DISJ")) {
                    list.addAll(((Disjunction) c).getDrs1().getAllPredicatesByType(type));
                    list.addAll(((Disjunction) c).getDrs2().getAllPredicatesByType(type));
                } else {
                    list.addAll(((CmplxStruct) c).getNestedDRS().getAllPredicatesByType(type));
                }
            }
        }
        return list;
    }
    /**
     * Returns all predicates in this DRS and inside the nested ones with a given type
     * @param ref
     * @return
     */
    public ArrayList<Predicate> getAllPredicatesByRef(String ref) {
        ArrayList<Predicate> list = new ArrayList<Predicate>();
        for (Condition c: conditions) {
            if (c instanceof Predicate) {
                Predicate p = (Predicate) c;
                if (p.getRef().equals(ref)) list.add((Predicate) c);
            } else {
                if (c.getType().equals("DISJ")) {
                    list.addAll(((Disjunction) c).getDrs1().getAllPredicatesByRef(ref));
                    list.addAll(((Disjunction) c).getDrs2().getAllPredicatesByRef(ref));
                } else {
                    list.addAll(((CmplxStruct) c).getNestedDRS().getAllPredicatesByRef(ref));
                }
            }
        }
        return list;
    }
    /**
     * Returns the first condition with the given type
     * @param type
     * @return
     */
    public Condition getConditionByType(String type) {

        for (Condition c: conditions) {
            if (c.getType().equals(type)) return c;
        }
        return null;
    }

    @Override
    public String toString() {
        return "drs(" + domain + "," + conditions + ')';
    }



}
